package com.capitalware.mqcm.mq;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertStore;
import java.security.cert.LDAPCertStoreParameters;
import java.util.ArrayList;
import java.util.Hashtable;

import com.capitalware.mqcm.objects.MQObject;
import com.ibm.mq.MQException;
import com.ibm.mq.MQQueueManager;
import com.ibm.mq.MQSecurityExit;
import com.ibm.mq.constants.CMQC;

/**
 * A base class to define MQ connection and disconnection methods.
 *
 * @author Roger Lacroix, Capitalware Inc.
 * @version 1.2
 * @license Apache 2 License
 */
public class MyMQ extends Thread
{
   protected   MQObject        qm;
   protected   MQQueueManager  _qMgr = null;

   /**
    * The constructor
    * @param logger
    */
   public MyMQ(MQObject qm)
   {
      super();
      this.qm = qm;
   }

   /**
    * Connect to MQ Server
    *
    * @throws MQException
    * @throws MalformedURLException
    * @throws ClassNotFoundException
    * @throws IllegalAccessException
    * @throws InstantiationException
    * @throws NoSuchAlgorithmException
    * @throws InvalidAlgorithmParameterException
    */
   public void connect() throws MQException, MalformedURLException, ClassNotFoundException, InstantiationException, IllegalAccessException, InvalidAlgorithmParameterException, NoSuchAlgorithmException
   {
      MQException.log = null;

      if (qm.channelName != null)
      {
         Hashtable<String, Object> mqht = new Hashtable<String, Object>();

         mqht.put(CMQC.CHANNEL_PROPERTY, qm.channelName);
         mqht.put(CMQC.HOST_NAME_PROPERTY, qm.hostname);
         mqht.put(CMQC.PORT_PROPERTY, new Integer(qm.port));

         if (qm.userID != null)
            mqht.put(CMQC.USER_ID_PROPERTY, qm.userID);

         if (qm.password != null)
            mqht.put(CMQC.PASSWORD_PROPERTY, qm.password);

         /**
          * Tell MQ to use MQCSP structure for UserId & Password
          */
         if ( (!("".equals(qm.userID))) && (qm.userID != null) && 
              (!("".equals(qm.password))) && (qm.password != null)  )
          {
             mqht.put(CMQC.USE_MQCSP_AUTHENTICATION_PROPERTY, true );
             
             /* Force the JVM UserId to be what the user has set because MQCSP 
              * picks up the JVM UserId. */
             if (mqht.get(CMQC.USER_ID_PROPERTY) != null)
                System.setProperty("user.name", (String)mqht.get(CMQC.USER_ID_PROPERTY));
          }
          else
          {
             mqht.put(CMQC.USE_MQCSP_AUTHENTICATION_PROPERTY, false );
          }

         if (qm.securityExit != null)
         {
            Class se = getSEClass(qm.securityExit, qm.securityExitPath);
            mqht.put(CMQC.SECURITY_EXIT_PROPERTY, (MQSecurityExit) se.newInstance());
         }

         /*
          * SSL
          */
         if (qm.cipherSuiteName != null)
            mqht.put(CMQC.SSL_CIPHER_SUITE_PROPERTY, qm.cipherSuiteName);

         if (qm.distinguishedName != null)
            mqht.put(CMQC.SSL_PEER_NAME_PROPERTY, qm.distinguishedName);

         if (qm.trustedStore != null)
            System.setProperty("javax.net.ssl.trustStore", qm.trustedStore);

         if (qm.trustedStorePasswd != null)
            System.setProperty("javax.net.ssl.trustStorePassword", qm.trustedStorePasswd);

         if (qm.keyStore != null)
            System.setProperty("javax.net.ssl.keyStore", qm.keyStore);

         if (qm.keyStorePasswd != null)
            System.setProperty("javax.net.ssl.keyStorePassword", qm.keyStorePasswd);

         if ( ((qm.ldapServer != null)) && (qm.ldapServerPort != 0) )
         {
            LDAPCertStoreParameters ldapcsparms = new LDAPCertStoreParameters(qm.ldapServer, qm.ldapServerPort);
            CertStore cs = CertStore.getInstance("LDAP", ldapcsparms);
            ArrayList<Object> al = new ArrayList<Object>();
            al.add(cs);
            mqht.put(CMQC.SSL_CERT_STORE_PROPERTY, al);
         }

         _qMgr = new MQQueueManager(qm.qMgrName, mqht);
      }
      else if (qm.CCDTFile != null)
      {
         String trustedStore = System.getenv("MQSSLKEYR");
         if (trustedStore != null)
            System.setProperty("javax.net.ssl.trustStore", trustedStore + ".jks");

         File file = new File(qm.CCDTFile);
         URL url = file.toURI().toURL();

         _qMgr = new MQQueueManager(qm.qMgrName, url);
      }
      else
      {
         _qMgr = new MQQueueManager(qm.qMgrName);
      }
   }

   /**
    *
    * @param clazzName
    * @param pathToJar
    * @return
    */
   private Class getSEClass(String clazzName, String pathToJar) throws MalformedURLException, ClassNotFoundException
   {
      Class cls = null;

      // Create a File object on the root of the directory containing the class file
      File file = new File(pathToJar);

      // Convert File to a URL
      URL url = file.toURI().toURL();
      URL[] urls = new URL[]
      {
         url
      };

      // Create a new class loader with the directory
      ClassLoader cl = new URLClassLoader(urls);

      // Load in the class; MyClass.class should be located in
      // the directory file:/c:/myclasses/com/mycompany
      cls = cl.loadClass(clazzName);

      return cls;
   }

   /**
    * Disconnect from the queue manager
    */
   public void disconnect()
   {
      try
      {
         if (_qMgr != null)
            _qMgr.disconnect();
      }
      catch (MQException e)
      {}
   }
}
